<?php
/**
 * @file uprpc.admin.inc
 * Contains admin menu callbacks.
 */

/**
 * Menu callback.
 */
function uprpc_settings() {
  $form = array();

  $form['db_connection'] = array(
    '#type' => 'fieldset',
    '#title' => t('Remote site details'),
    '#collapsible' => TRUE,
  );

  // TODO: this is obsolete.
  $form['db_connection']['uprpc_database'] = array(
    '#type' => 'textfield',
    '#title' => t('Database URL'),
    '#description' => t("A Drupal database URL for the import database, in the same format as in settings.php. " .
      "Note that this should be in the D6 format, even though it is a D5 database. " .
      "Example: 'mysqli://username:password@localhost/database'."),
    '#default_value' => variable_get('uprpc_database', 'mysqli://username:password@localhost/database'),
    '#size' => 60,
    '#maxlength' => 128,
    '#required' => TRUE,
  );
  
  $form['services_connection'] = array(
    '#type' => 'fieldset',
    '#title' => t('Services connection'),
    '#collapsible' => TRUE,
  );
  
  $form['services_connection']['uprpc_server'] = array(
    '#type' => 'textfield',
    '#title' => t('Server URL'),
    '#description' => t("The base URL on the remote server; that is, the Drupal root. Include the trailing slash, eg, 'http://example.com/'"),
    '#default_value' => variable_get('uprpc_server', ''),
    '#size' => 60,
    '#maxlength' => 128,
    '#required' => TRUE,
  );
  $form['services_connection']['uprpc_path'] = array(
    '#type' => 'textfield',
    '#title' => t('Server path'),
    '#description' => t("The path to the remote site's XMLRPC. This is probably 'services/xmlrpc'"),
    '#default_value' => variable_get('uprpc_path', 'services/xmlrpc'),
    '#size' => 60,
    '#maxlength' => 128,
    '#required' => TRUE,
  );
  $form['services_connection']['uprpc_services_api_key'] = array(
    '#type' => 'textfield',
    '#title' => t('API key'),
    '#description' => t("The API key generated by Services module on the Drupal 5 site."),
    '#default_value' => variable_get('uprpc_services_api_key', ''),
    '#size' => 60,
    '#maxlength' => 128,
    '#required' => TRUE,
  );
  $form['services_connection']['uprpc_webservice_username'] = array(
    '#type' => 'textfield',
    '#title' => t('Remote username'),
    '#description' => t('The name of a user on the Drupal 5 site. This user should have access to the required services on that site.'),
    '#default_value' => variable_get('uprpc_webservice_username', ''),
    '#size' => 60,
    '#maxlength' => 128,
    '#required' => TRUE,
  );
  $form['services_connection']['uprpc_webservice_pass'] = array(
    '#type' => 'textfield',
    '#title' => t('Remote password'),
    '#description' => t("The user's password."),
    '#default_value' => variable_get('uprpc_webservice_pass', ''),
    '#size' => 60,
    '#maxlength' => 128,
    '#required' => TRUE,
  );

  $form['content'] = array(
    '#type' => 'fieldset',
    '#title' => t('Content to import'),
    '#collapsible' => TRUE,
  );

  $form['content']['uprpc_vids'] = array(
    '#type' => 'textfield',
    '#title' => t('Vocabularies'),
    '#description' => t("Specify the IDs of vocabularies to import, as a comma-separated list. Example: '1,3,4'."),
    '#default_value' => variable_get('uprpc_vids', ''),
    '#size' => 60,
    '#maxlength' => 128,
  );
  
  // Until we have default handling for node types that aren't listed in hook_nodeimport implementations,
  // we might as well only list the ones we know how to handle!
  // @todo: fix default handling for unknown node types!
  $supported_node_types = array_keys(module_invoke_all('nodeimport', 'the $op param does nothing -- @todo'));
  if (count($supported_node_types)) {
    $node_type_options = array_combine($supported_node_types, $supported_node_types);
  }
  else {
    $node_type_options = array();
  }
  $form['content']['uprpc_types'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Content types'),
    '#description' => t("Specify the content types to import. The machine names of types on the remote site which are supported for migration are shown."),
    '#default_value' => variable_get('uprpc_types', array()),
    '#options' => $node_type_options,
  );

  $form['import_settings'] = array(
    '#type' => 'fieldset',
    '#title' => t('Import settings'),
    '#collapsible' => TRUE,
  );

  $form['import_settings']['uprpc_remote_mark'] = array(
    '#type' => 'checkbox',
    '#title' => t('Mark imported items on the remote site'),
    '#description' => t("Once an item is imported, it receives a flag on the remote site to mark this. Uncheck this to have all items imported (and hence updated) every time the import is run."),
    '#default_value' => variable_get('uprpc_remote_mark', TRUE),
  );

  return system_settings_form($form);
}

